class PipelineRunnerJob < ApplicationJob
  attr_accessor :event, :pipeline, :definition

  queue_as :default

  # receives a pipeline as it's argument
  def perform(pipeline_object, event=nil, definition_name=nil)

    if event
      event = Event.new(event)
      # this is the starting point for the pipeline if the event is present
      raise Exception.new('Wrong code path') if pipeline_object.present? || definition_name.nil?

      unless definition_name.constantize.trigger_when?(event)
        return
      end

      pipeline_object = Pipeline.create!(
          project: event.project,
          status: 'STARTING',
          event: event.data,
          definition: definition_name,
      )
    end

    self.pipeline = pipeline_object
    self.definition = pipeline.definition_class.new(pipeline, event)
    self.event = event || pipeline.event

    if pipeline.status?(:failed) || pipeline.status?(:complete)
      # pipeline has already been marked as failing or complete
      return
    end

    if last_stage_failed?
      # fail the pipeline if it's failing
      update_status :failed
      return
    end

    if last_stage_pending?
      # nothing to do here, still waiting on some methods to come through
      return
    end

    # if we've gotten here all the checks are passing and we can move onto the next stage

    unless next_stage.present?
      # no next stage, pipeline is complete
      complete_pipeline
      return
    end

    # run the next stage
    pipeline.update!(stage: next_stage, status: 'PENDING')
    execute(next_stage)

    unless last_stage_pending?
      # no jobs were created during the last stage, therefore everything is done!
      complete_pipeline
    end
  end

  def execute(this_stage)
    begin
      definition.send(this_stage)
    rescue Exception => e
      pipeline.update!(status: 'ERROR', error: e.message)
      raise e
    ensure
      pipeline.save
    end
  end

  def complete_pipeline
    definition.class.ensure_stages.each do |ensured_stage|
      # will fail the job if an error is raised
      execute(ensured_stage)
    end
    update_status :complete
  end

  def next_stage
    @next_stage ||= begin
      if pipeline.stage
        cur_idx = definition.class.stages.find_index { |st| st == pipeline.stage.downcase.to_sym }
        definition.class.stages[cur_idx + 1]
      else
        definition.class.stages[0]
      end
    end
  end

  def update_status(status)
    pipeline.update!(status: status.to_s.upcase)
  end

  def last_stage_failed?
    Job.where('pipeline_id = ? AND pipeline_stage = ? AND (status = "FAILED" OR status = "CANCELLED")', pipeline.id, pipeline.stage).exists?
  end

  def last_stage_pending?
    Job.where('pipeline_id = ? AND pipeline_stage = ? AND (status = "QUEUED" OR status = "PENDING")', pipeline.id, pipeline.stage).exists?
  end

end
